<!-- Global Javascript values -->

<script type="text/javascript">
    "use strict";
    /* Consts */
    const DEFAULT_LENGTH_MENU = [[10, 25, 50, 100 -1], [10, 25, 50, 100, "All"]];
    const DEFAULT_PAGE_LENGTH = 50;
</script>
