# Generated by Django 4.2.16 on 2025-02-19 10:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0010_alter_eveindustryactivityduration_eve_type_and_more"),
        ("dens", "0002_alter_mercenarydenreinforcednotification_den"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mercenaryden",
            name="location",
            field=models.OneToOneField(
                help_text="Location of the den",
                on_delete=django.db.models.deletion.CASCADE,
                to="eveuniverse.eveplanet",
            ),
        ),
    ]
