# Generated by Django 4.2.16 on 2025-01-30 11:50

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("dens", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mercenarydenreinforcednotification",
            name="den",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="dens.mercenaryden",
            ),
        ),
    ]
