{% load i18n %}

<li class="nav-item">
    <a class="nav-link" href="{% url 'dens:index' %}">
        <i class="fa-solid fa-tents"></i>
        <span class="d-md-inline m-2">{% translate "Dens" %}</span>
    </a>
</li>
